@@ -1,7 +1,7 @@
 #coding: utf-8
 from base64 import urlsafe_b64encode
 import json
-from flask import request, url_for, session, jsonify
+from flask import request, url_for, session
 from flask import current_app
 from flask_restful import Resource
 from flask_login import current_user, login_required
@@ -9,6 +9,7 @@ from flask_uploads import UploadNotAllowed
 import os
 import hashlib
 import pycountry 
+import ipaddress
 from datetime import date, datetime, timedelta
 from slugify import slugify 
 from loguru import logger
@@ -97,7 +98,7 @@ def get_tags(item):
 def process_casino(item, actors={}, check_commit=False):
     _doc = item['_source']
     _doc['id'] = item['_id']
-    _doc['full_path'] = f"https://{current_app.config['DOMAIN']}{get_page_url(_doc)}"
+    _doc['full_path'] = f"{current_app.config['PREFERRED_URL_SCHEME']}://{current_app.config['DOMAIN']}{get_page_url(_doc)}"
     if _doc.get('owner') and len(actors.keys()):
         _doc['owner'] = {'id': _doc['owner'], 'username': actors.get(_doc['owner'])}
 
@@ -385,8 +386,23 @@ class ApiResourceSave(Resource):
                     if entity['category'] not in ['provider']:
                         entity['geo'] = []
                         
-                    resp, _ = Page.put(_id, entity)
-                    logger.info(f'Page saved: {resp}')
+
+                    if current_app.config.get('CORE_PREFIX') and entity['category'] in ['page', 'collection'] and entity.get('bulk_create') and not resource_key:
+                        _locales = entity['locale_available']
+                        entity['locale_available'] = []
+                        for _locale in _locales:
+                            pages, found = Page.get(path=path, locale=_locale)
+                            if found == 0:
+                                entity['locale'] = _locale
+                                resp, _ = Page.put(Page.generate_id(path, _locale), entity)
+                                logger.info(f'Page with locale {_locale} saved: {resp}')
+                            else:
+                                page = pages.pop()
+                                logger.info(f'Page with locale {_locale} already exists: {page.title}')
+                    else:
+                        resp, _ = Page.put(_id, entity)
+                        logger.info(f'Page saved: {resp}')
+
                     return {'ok': True}
 
         return {'ok': False, 'errors': _errors}
@@ -1016,7 +1032,7 @@ class ApiAdminUsersGet(Resource):
                     'admin',
                     'content',
                     'tds',
-                    'bots',
+                    'telegram',
                     'tools',
                 ],
                 'total': total                
@@ -1235,13 +1251,15 @@ class ApiContentPages(Resource):
                 'meta_title',
                 'collection_mode',
                 'locale',
+                'tags',
             ]
         )
 
         def process_page(item):
             _doc = item['_source']
             _doc['id'] = item['_id']
-            _doc['full_path'] = f"https://{current_app.config['DOMAIN']}{get_page_url(_doc)}"
+            _doc['title'] = _doc['title'] or _doc['path']
+            _doc['full_path'] = f"{current_app.config['PREFERRED_URL_SCHEME']}://{current_app.config['DOMAIN']}{get_page_url(_doc)}"
             return _doc
 
         return {
@@ -1308,7 +1326,7 @@ class ApiContentSlots(Resource):
         def process_page(item):
             _doc = item['_source']
             _doc['id'] = item['_id']
-            _doc['full_path'] = f"https://{current_app.config['DOMAIN']}{get_page_url(_doc)}"
+            _doc['full_path'] = f"{current_app.config['PREFERRED_URL_SCHEME']}://{current_app.config['DOMAIN']}{get_page_url(_doc)}"
             return _doc
 
         return {
@@ -1419,7 +1437,7 @@ class ApiContentFeedback(Resource):
             if len(item['_source']['comments']):
                 _d2 = {
                     'casino': item['_source']['title'], 
-                    'full_path': f"https://{current_app.config['DOMAIN']}{get_page_url(item['_source'])}"
+                    'full_path': f"{current_app.config['PREFERRED_URL_SCHEME']}://{current_app.config['DOMAIN']}{get_page_url(item['_source'])}"
                 }
                 _approved += [process_approved(dict(_d1, **_d2)) for _d1 in item['_source']['comments']]
 
@@ -2055,6 +2073,13 @@ class ApiTdsStats(Resource):
         items = []
         total = 0
         aggs_name = None
+        aggs_fields = ['campaign_alias', 'campaign_name', 'stream_name', 'stream', 'country', 'is_bot', 'is_uniq']
+
+        kwargs = {}
+        if 'filters' in data:
+            for attr in aggs_fields:
+                if attr in data['filters'] and data['filters'][attr]:
+                    kwargs[attr] = data['filters'][attr]
 
         if data['mode'] == 'Clicks':
             _sort = [{'createdon': {'order': 'desc'}}]
@@ -2065,6 +2090,7 @@ class ApiTdsStats(Resource):
                 _sort = [{'createdon': {'order': 'asc'}}]
 
             items, total = TdsHit.get(
+                **kwargs,
                 _sort=_sort,
                 _process=False,
                 _count=cpp,
@@ -2130,9 +2156,19 @@ class ApiTdsStats(Resource):
                 field = 'action'
 
             if field:
-                items, total = TdsHit.aggs(field, _range=('createdon', start, end, _d.get('timezone', 'UTC')))
+                items, total = TdsHit.aggs_stats(
+                    field, 
+                    _range=('createdon', start, end, _d.get('timezone', 'UTC')),
+                    _filters=kwargs
+                )
 
-        return {'ok': True, 'items': items, 'total': total, 'aggs_name': aggs_name}
+        return {
+            'ok': True, 
+            'items': items, 
+            'total': total, 
+            'aggs_name': aggs_name,
+            'filters': TdsHit.aggs(aggs_fields, _range=('createdon', start, end, _d.get('timezone', 'UTC')))
+        }
 
 
 class ApiTdsCampaigns(Resource):
@@ -2146,7 +2182,12 @@ class ApiTdsCampaigns(Resource):
         if data['sorting'] in ['name', 'alias']:
             _sort = [{'{0}.keyword'.format(data['sorting']): {'order': 'asc'}}]
 
+        kwargs = {}
+        if 'filters_group' in data['payload'] and data['payload']['filters_group']:
+            kwargs['groups'] = data['payload']['filters_group']
+
         campaigns, _ = TdsCampaign.get(
+            **kwargs,
             _all=True,
             _sort=_sort,
             _process=False,
@@ -2164,7 +2205,7 @@ class ApiTdsCampaigns(Resource):
         )
 
         start, end = get_timerange(data['payload']['range'], data['payload']['custom_range'])
-        stats, _ = TdsHit.aggs('stream', _range=('createdon', start, end, data.get('timezone', 'UTC')))
+        stats, _ = TdsHit.aggs_stats('stream', _range=('createdon', start, end, data.get('timezone', 'UTC')))
         stats = {item['term']: {'hits': item['hits'], 'uc': item['uc']} for item in stats}
 
         def process_campaign(item):
@@ -2173,7 +2214,12 @@ class ApiTdsCampaigns(Resource):
             _doc['url'] = '{0}{1}'.format(_doc['domain'], _doc['alias'])
             return _doc
 
-        return {'ok': True, 'items': [process_campaign(item) for item in campaigns], 'stats': stats}
+        return {
+            'ok': True, 
+            'items': [process_campaign(item) for item in campaigns], 
+            'stats': stats,
+            'filters': TdsCampaign.aggs(['groups'])
+        }
 
 
 class ApiTdsCampaignsGet(Resource):
@@ -2242,9 +2288,16 @@ class ApiTdsCampaignsGet(Resource):
                 {'key': 'remote', 'value': 'Remote URL'},
                 {'key': 'campaign', 'value': 'Send to Campaign'},
                 {'key': 'html', 'value': 'Show as HTML'},
+            ],
+            'processors': [
+                'softswiss',
+                'pelican'
             ]
         }
 
+        if data:
+            _doc['postback_url'] = f"{_doc['domain']}postback/{data['id']}"
+
         return {'ok': True, 'options': _opts, 'campaign': _doc}
 
 
@@ -2493,6 +2546,28 @@ class ApiTdsBotsClear(Resource):
         return {'ok': True}
 
 
+class ApiTdsBotsDownload(Resource):
+    @login_required
+    @zone_required(zone='tds')
+    def post(self):
+        data = request.get_json()
+        logger.info(u'Data ApiTdsBotsDownload: {0}'.format(data))
+        _data = current_app.redis.smembers(f'tds_bots_{data["section"]}')
+
+        def process_item(s):
+            s = s.decode()
+            if data["section"] == 'ip' and s.isdigit():
+                s = ipaddress.ip_address(int(s))
+            return str(s)
+
+        return {
+            'ok': True, 
+            'content': '\n'.join(sorted([process_item(item) for item in list(_data)])), 
+            'content_type': 'plain/text', 
+            'filename': f'export-{data["section"]}-{len(_data)}.txt'
+        }
+
+
 class ApiTdsCampaignUniqClear(Resource):
     @login_required
     @zone_required(zone='tds')
