@@ -42,7 +42,7 @@ class LicencesExtension(Extension):
             is_active=True,
             is_searchable=True,
             is_redirect=False,  
-            locale=current_app.config['BABEL_DEFAULT_LOCALE'], 
+            locale=g.language, 
             _source=['title', 'path', 'alt_title'],
             collection_mode='CasinoLicence',
             _count=100,
@@ -75,7 +75,7 @@ class SoftwaresExtension(Extension):
             is_active=True,
             is_searchable=True,
             is_redirect=False,  
-            locale=current_app.config['BABEL_DEFAULT_LOCALE'], 
+            locale=g.language, 
             _source=['title', 'path', 'alt_title', 'custom_text', 'logo'],
             collection_mode='CasinoSoftware',
             _count=100,
@@ -94,7 +94,7 @@ class SoftwaresExtension(Extension):
             {"term": {"is_searchable": True}},
             {"term": {"is_redirect": False}},
             {"term": {"category.keyword": 'provider'}},
-            {"term": {"locale.keyword": current_app.config['BABEL_DEFAULT_LOCALE']}},
+            {"term": {"locale.keyword": g.language}},
         ]
 
         _aggs = {
@@ -165,7 +165,7 @@ class CountriesExtension(Extension):
             is_active=True,
             is_searchable=True,
             is_redirect=False,  
-            locale=current_app.config['BABEL_DEFAULT_LOCALE'], 
+            locale=g.language, 
             _source=['title', 'path', 'alt_title'],
             collection_mode='CasinoCountry',
             _count=100,
@@ -198,7 +198,7 @@ class BankingExtension(Extension):
             is_active=True,
             is_searchable=True,
             is_redirect=False,  
-            locale=current_app.config['BABEL_DEFAULT_LOCALE'], 
+            locale=g.language, 
             _source=['title', 'path', 'alt_title', 'custom_text', 'logo'],
             collection_mode='CasinoBanking',
             _count=100,
@@ -217,7 +217,7 @@ class BankingExtension(Extension):
             {"term": {"is_searchable": True}},
             {"term": {"is_redirect": False}},
             {"term": {"category.keyword": 'provider'}},
-            {"term": {"locale.keyword": current_app.config['BABEL_DEFAULT_LOCALE']}},
+            {"term": {"locale.keyword": g.language}},
         ]
 
         _aggs = {
@@ -314,7 +314,7 @@ class RatingExtension(Extension):
             country=current_user.country_full,
             services=service,
             provider_tags=tags,
-            _locale=current_app.config['BABEL_DEFAULT_LOCALE'],
+            _locale=g.language,
             _source = [
                 "title", 
                 "alias", 
@@ -374,36 +374,36 @@ class RatingExtension(Extension):
         return CallBlock(self.call_method("_render", args=[List(args)]), [], [], []).set_lineno(lineno)          
 
 
-class SlotsExtension(Extension):
-    tags = set(['slots'])
+# class SlotsExtension(Extension):
+#     tags = set(['slots'])
 
-    def __init__(self, environment):
-        super(SlotsExtension, self).__init__(environment)
+#     def __init__(self, environment):
+#         super(SlotsExtension, self).__init__(environment)
 
-    def _render(self, caller):
-        slots, found = Page.get(
-            category='slot', 
-            is_active=True, 
-            is_searchable=True, 
-            is_redirect=False, 
-            locale=current_app.config['BABEL_DEFAULT_LOCALE'], 
-            _source=['alias', 'title', 'cover', 'software'], 
-            _count=500,
-            _sort=[
-                {'title.keyword': {'order': 'asc'}}
-            ]
-        )
-        t = current_app.jinja_env.get_template('_slots.html')
-        s = t.render(slots=slots, found=found)
-        return s
+#     def _render(self, caller):
+#         slots, found = Page.get(
+#             category='slot', 
+#             is_active=True, 
+#             is_searchable=True, 
+#             is_redirect=False, 
+#             locale=g.language, 
+#             _source=['alias', 'title', 'cover', 'software'], 
+#             _count=500,
+#             _sort=[
+#                 {'title.keyword': {'order': 'asc'}}
+#             ]
+#         )
+#         t = current_app.jinja_env.get_template('_slots.html')
+#         s = t.render(slots=slots, found=found)
+#         return s
 
-    def parse(self, parser):
-        lineno = next(parser.stream).lineno
+#     def parse(self, parser):
+#         lineno = next(parser.stream).lineno
 
-        body = ''
-        args = []
+#         body = ''
+#         args = []
 
-        return CallBlock(self.call_method("_render", args), [], [], body).set_lineno(lineno)         
+#         return CallBlock(self.call_method("_render", args), [], [], body).set_lineno(lineno)         
 
 
 class TopSlotsExtension(Extension):
@@ -430,7 +430,7 @@ class TopSlotsExtension(Extension):
             is_active=True, 
             is_searchable=True, 
             is_redirect=False,
-            locale=current_app.config['BABEL_DEFAULT_LOCALE'], 
+            locale=g.language, 
             _source=True, 
             _count=count,
             _random=random,
@@ -438,7 +438,7 @@ class TopSlotsExtension(Extension):
             software=provider, 
         )
 
-        t = current_app.jinja_env.get_template('_slots_list.html')
+        t = current_app.jinja_env.get_template('_ext-slots_list.html')
         s = t.render(
             slots=slots,
             in_row=in_row,
@@ -470,7 +470,7 @@ class GuideExtension(Extension):
             is_searchable=True, 
             is_redirect=False, 
             tags=args[0],
-            locale=current_app.config['BABEL_DEFAULT_LOCALE'], 
+            locale=g.language, 
             _source=['path', 'title'], 
             _count=500,
             _sort=[
@@ -503,7 +503,7 @@ class TopCasinosExtension(Extension):
             country=current_user.country_full,
             is_searchable=True,
             is_redirect=False,
-            _locale=current_app.config['BABEL_DEFAULT_LOCALE'],
+            _locale=g.language,
             _source = [
                 "title", 
                 "alias", 
@@ -638,7 +638,7 @@ class TopAlexaExtension(Extension):
             is_active=True,
             is_searchable=True,
             is_redirect=False,  
-            locale=current_app.config['BABEL_DEFAULT_LOCALE'], 
+            locale=g.language, 
             _source=['title', 'path', 'alt_title', 'rank_alexa'],
             _count=30,
             _sort=[
@@ -716,7 +716,7 @@ class CasinosExtension(Extension):
                 tags='casino',
                 provider_tags=tags,
                 locale_available=g.language,
-                _locale=current_app.config['BABEL_DEFAULT_LOCALE'],
+                _locale=g.language,
                 _source=True, 
                 _count=count,
             )
@@ -761,7 +761,7 @@ class SlotsExtension(Extension):
 
         count = int(args[0])
         template = args[1]
-        has_filters = args[2]
+        has_filters = bool(args[2])
 
         tags = args[3:]
         logger.info('Slots Tags: {0}'.format(tags))
@@ -769,21 +769,23 @@ class SlotsExtension(Extension):
         if has_filters:
             pass
         else:
-            pages, found = Page.provider_by_context(
-                is_searchable=True,
-                is_redirect=False,
-                country=current_user.country_full,
-                tags='casino',
-                provider_tags=tags,
-                locale_available=g.language,
-                _locale=current_app.config['BABEL_DEFAULT_LOCALE'],
-                _source=True, 
-                _count=count,
-            )
+            slots, found = Page.get(
+                category='slot', 
+                is_active=True, 
+                is_searchable=True, 
+                is_redirect=False, 
+                locale=g.language, 
+                # locale_available=g.language,
+                _source=['alias', 'title', 'cover', 'software'], 
+                _count=500,
+                _sort=[
+                    {'title.keyword': {'order': 'asc'}}
+                ]
+            )            
 
         t = current_app.jinja_env.get_template('_ext-{0}.html'.format(template))
         s = t.render(
-            pages=pages, 
+            slots=slots, 
             found=found, 
             has_filters=has_filters, 
             count=count,
@@ -884,17 +886,7 @@ class GeoCountriesExtension(Extension):
 
     def _render(self, caller):
         t = current_app.jinja_env.get_template('_ext-geo_countries.html')
-        countries = [
-            ('United Kingdom', 'gb'),
-            ('Germany', 'de'),
-            ('Sweden', 'se'),
-            ('Norway', 'no'),
-            ('Spain', 'es'),
-            ('Portugal', 'pt'),
-            ('Canada', 'ca'),
-            ('Australia', 'au'),
-            ('New Zealand', 'nz'),
-        ]
+        countries = g.countries
         if not current_user.location_iso in [i for c, i in countries]:
             countries = [(current_user.location, current_user.location_iso)] + countries
 
@@ -950,7 +942,7 @@ class ComparatorExtension(Extension):
                     is_active=True,
                     is_searchable=True,
                     is_redirect=False,  
-                    locale=current_app.config['BABEL_DEFAULT_LOCALE'], 
+                    locale=g.language, 
                     _count=3,
                 )
 
@@ -1070,6 +1062,14 @@ class BlogExtension(Extension):
     def _render(self, args, caller=None):
         logger.info('Args: {0}'.format(args))
 
+        default = [12, 3, True]
+        _ca = len(default)
+        args = args[:_ca] + default[len(args):] + args[_ca:]
+
+        _cpp = int(args[0])
+        _row = int(args[1])
+        _paging = bool(args[2])
+
         page = 1
         p = request.args.get('page')
         if p:
@@ -1078,12 +1078,11 @@ class BlogExtension(Extension):
             else:
                 abort(404)
 
-        _cpp = 12
         pages, total = Page.get(
             is_active=True,
             is_searchable=True, 
             is_redirect=False, 
-            locale=current_app.config['BABEL_DEFAULT_LOCALE'], 
+            locale=g.language, 
             tags='Blog',
             _source=['path', 'title', 'publishedon', 'cover'],
             _count=_cpp,
@@ -1099,10 +1098,12 @@ class BlogExtension(Extension):
         if (page - 1) * _cpp > total or (request.args.get('page') and page < 2):
             abort(404)
 
-        paging = [{'page': item, 'value': item} for item in range(1, math.ceil(total / _cpp) + 1)]
+        paging = []
+        if _paging:
+            paging = [{'page': item, 'value': item} for item in range(1, math.ceil(total / _cpp) + 1)]
 
         t = current_app.jinja_env.get_template('_ext-blog.html')
-        return t.render(pages=pages, paging=paging, current=page)
+        return t.render(pages=pages, paging=paging, current=page, row=_row)
 
     def parse(self, parser):
         lineno = next(parser.stream).lineno
