@@ -277,9 +277,52 @@ class ElasticEntity(BaseEntity):
             cls.on_delete.send(response=resp)
         return resp
 
-    # @classmethod
-    # def process_response(cls, _items):
-    #     return [dict(item.to_dict(), **dict(_id=item._id)) for item in _items]
+    @classmethod
+    def aggs(cls, fields=[], _range=None, _cnt=500):
+
+        aggs = {
+            item: {
+                "terms": {
+                    "field": f'{item}.keyword',
+                    "size": _cnt
+                }
+            } for item in fields
+        }        
+
+        must = []
+
+        # if _filters:
+        #     def pre_field(k, v):
+        #         return k if isinstance(v, bool) else '{0}.keyword'.format(k)
+        #     must = [{"terms" if isinstance(v, list) else "term": {pre_field(k, v): v}} for k, v in _filters.items()]
+
+        if _range:
+            _f, _s, _e, _tz = _range
+            must.append({"range": {_f: {"gte": _s, "lte": _e, "time_zone": _tz}}})
+
+        q = {
+            "query": {
+                "bool": {
+                    "must": must
+                }
+            },
+            "size": 0,
+            "aggs": aggs
+        }
+
+        resp = current_app.es.search(index=cls._table(), body=q, request_timeout=60, ignore=[400, 404])
+
+        aggs = {}
+        if 'status' in resp and resp['status'] in [400, 404]:
+            pass
+        else:
+            aggs = resp.get('aggregations', [])            
+
+        def process(items):
+            return sorted([item['key'].strip() for item in items['buckets'] if item['key'].strip()])
+
+        opts = {k: process(v) for k, v in aggs.items()}
+        return opts
 
 
 class Page(ElasticEntity):
@@ -620,7 +663,7 @@ class Page(ElasticEntity):
 
         def process(items, key):
             if key in items:
-                return [item['key'].strip() for item in items[key]['buckets'] if item['key'].strip()]
+                return sorted([item['key'].strip() for item in items[key]['buckets'] if item['key'].strip()])
             return []
 
         opts = {
@@ -632,8 +675,8 @@ class Page(ElasticEntity):
             'provider_cons': process(aggs, 'provider_cons'),
             'software': process(aggs, 'software'),
             'licences': process(aggs, 'licences'),
-            'languages': list(set(process(aggs, 'languages') + process(aggs, 'support_languages'))),
-            'payment_methods': list(set(process(aggs, 'deposits') + process(aggs, 'withdrawal'))),
+            'languages': sorted(list(set(process(aggs, 'languages') + process(aggs, 'support_languages')))),
+            'payment_methods': sorted(list(set(process(aggs, 'deposits') + process(aggs, 'withdrawal')))),
             'games': process(aggs, 'games'),
 
             'themes': process(aggs, 'themes'),
@@ -1063,7 +1106,7 @@ class TdsHit(ElasticEntity):
         return "{0}_tds_hits.{1}".format(os.environ.get('PROJECT', 'project'), os.environ.get('VERSION', 'v1'))
 
     @classmethod
-    def aggs(cls, _field, _range=None, _cnt=500):
+    def aggs_stats(cls, _field, _range=None, _filters={}, _cnt=500):
 
         def get_subaggs(_a={}):
             _b = {
@@ -1158,6 +1201,11 @@ class TdsHit(ElasticEntity):
 
         must = []
 
+        if _filters:
+            def pre_field(k, v):
+                return k if isinstance(v, bool) else '{0}.keyword'.format(k)
+            must = [{"terms" if isinstance(v, list) else "term": {pre_field(k, v): v}} for k, v in _filters.items()]
+
         if _range:
             _f, _s, _e, _tz = _range
             must.append({"range": {_f: {"gte": _s, "lte": _e, "time_zone": _tz}}})
@@ -1238,6 +1286,7 @@ class TdsCampaign(ElasticEntity):
         'is_archive': bool,
         'notes': str,
         'streams': list,
+        'postback_processor': str,
     }
         
     @classmethod
@@ -1245,6 +1294,33 @@ class TdsCampaign(ElasticEntity):
         return "{0}_tds_campaign.{1}".format(os.environ.get('PROJECT', 'project'), os.environ.get('VERSION', 'v1'))
 
 
+class TdsPostback(ElasticEntity):
+    on_put = Signal()
+
+    _schemas = {
+        'createdon': DateTimeField,
+        'postback_id': str,
+        'ip': str,
+        'ua': str,
+        'campaign_id': str,
+        'campaign_name': str,
+        'method': str,
+        'args': str,
+        'content_type': str,
+        'payload': str,
+        # 'click_id': str,
+        # 'action': str, # reg, fdep, dep
+        # 'player_id': str, # internal id
+        # 'amount': float,
+        # 'currency': str,
+        # 'payment_id': str,
+    }
+        
+    @classmethod
+    def _table(cls):
+        return "{0}_tds_postback.{1}".format(os.environ.get('PROJECT', 'project'), os.environ.get('VERSION', 'v1'))
+
+
 # pydantic models
 
 class UserModel(BaseModel):
@@ -1405,6 +1481,7 @@ class CampaignModel(BaseModel):
     is_archive: bool = False
     notes: Optional[str] = None
     streams: conlist(dict, min_items=1)
+    postback_processor: Optional[str] = None
 
 
 class ActivityTicketModel(BaseModel):
